/*

	Copyright  2003 Frank Condello
	
	Portions Copyright  2001 Dair Grant <mailto:dair@webthing.net>
	and Copyright  2003 Joe Strout <joe@strout.net> (from "LoadSaveJPEG")
	
	"GetData" based on code by Will Leshner <leshner@ljug.com>
	
	Linux port contributed by Jonathan Johnson <jonj@realsoftware.com>

*/

/* PNGuWin32.cpp: Win32 routines */

#include "PNGu.h"


/* ---------------------------------------------------------------------------
	OpenFILEFromFolderItem
	
	Open a FILE* for a FolderItem. Encoding trick lifted from here:
	http://support.realsoftware.com/listarchives/realbasic-plugins/2004-01/msg00051.html
---------------------------------------------------------------------------- */
FILE *OpenFILEFromFolderItem(REALfolderItem theFile, const char *fileMode)
{
	REALstring	path = NULL;
	FILE		*ansiFile = NULL;
	
	if (theFile)
	{
		path = REALConvertString( REALpathFromFolderItem(theFile), REALGetFontEncoding("System") );
		if (path) ansiFile = fopen(path->CString(), fileMode);
	}
	
	return ansiFile;
}


/* ---------------------------------------------------------------------------
	BuildImageDataFromPicture
	
	From LoadSaveJPEG: We fill imageData with packed RGB pixel values.
---------------------------------------------------------------------------- */
Boolean BuildImageDataFromPicture(REALpicture thePicture, UInt32 theWidth, UInt32 theHeight, UInt8 *imageData)
{
	UInt32			x, y, dstRowBytes;
	UInt8			*dstPtr;
	COLORREF		thePixel;
	HDC				theDC;
	REALgraphics	graphics;
	
	graphics = REALGetPictureGraphics(thePicture);
	if (graphics == NULL)
		return(false);
	
	// Get the DC for the picture
	theDC = REALGraphicsDC(graphics);	
	if (theDC == NULL)
	{
		REALUnlockObject((REALobject)graphics);
		return false;
	}
	
	// Extract the image data from the GWorld
	//
	// This is not going to give great performance on larger images, so if you
	// have some code to extract the pixels directly from an HDC let me know.
	dstPtr      = imageData;
	dstRowBytes = (theWidth * 3);

	for (y = 0; y < theHeight; y++)
	{
		// Copy another scanline
		for (x = 0; x < theWidth; x++)
		{
			// Grab the pixel
			thePixel = GetPixel(theDC, x, y);
			
			// Save it as RGB
			dstPtr[(x * 3) + 0] = ((thePixel >>  0) & 0x000000FF);
			dstPtr[(x * 3) + 1] = ((thePixel >>  8) & 0x000000FF);
			dstPtr[(x * 3) + 2] = ((thePixel >> 16) & 0x000000FF);
		}

		// Move on to the next scanline
		dstPtr += dstRowBytes;
	}
		
	REALUnlockObject((REALobject)graphics);
	
	return true;
}


/* ---------------------------------------------------------------------------
	BuildImageDataFromPictureAndMask
	
	Modified "BuildImageDataFromPicture" from LoadSaveJPEG
	FIXME! - Like the mac version, there's probably a way to do this in one shot
---------------------------------------------------------------------------- */
Boolean BuildImageDataFromPictureAndMask(REALpicture thePicture, REALpicture theMask, UInt32 theWidth, UInt32 theHeight, UInt8 *imageData)
{
	UInt32			x, y, dstRowBytes;
	UInt8			*dstPtr;
	COLORREF		thePixel;
	HDC				theDC;
	REALgraphics	graphics;
	
	graphics = REALGetPictureGraphics(thePicture);
	if (graphics == NULL)
		return false;
	
	// Get the DC for the picture
	theDC = REALGraphicsDC(graphics);	
	if (theDC == NULL)
	{
		REALUnlockObject((REALobject)graphics);
		return false;
	}

	// Extract the image data from the GWorld
	//
	// This is not going to give great performance on larger images, so if you
	// have some code to extract the pixels directly from an HDC let me know.
	dstPtr      = imageData;
	dstRowBytes = (theWidth * 4);

	for (y = 0; y < theHeight; y++)
	{
		// Copy another scanline
		for (x = 0; x < theWidth; x++)
		{
			// Grab the pixel
			thePixel = GetPixel(theDC, x, y);
			
			// Store it as RGBA
			dstPtr[(x * 4) + 0] = ((thePixel >>  0) & 0x000000FF);
			dstPtr[(x * 4) + 1] = ((thePixel >>  8) & 0x000000FF);
			dstPtr[(x * 4) + 2] = ((thePixel >> 16) & 0x000000FF);
			dstPtr[(x * 4) + 3] = 0;
		}

		// Move on to the next scanline
		dstPtr += dstRowBytes;
	}
	
	// Get the DC for the mask picture
	theDC = NULL;
	REALUnlockObject((REALobject)graphics);
	graphics = NULL;
	
	graphics = REALGetPictureGraphics(theMask);
	if (graphics == NULL)
		return false;
	
	// Get the DC for the picture
	theDC = REALGraphicsDC(graphics);	
	if (theDC == NULL)
	{
		REALUnlockObject((REALobject)graphics);
		return false;
	}
	
	dstPtr = &imageData[0];
	
	for (y = 0; y < theHeight; y++)
	{
		// Copy another scanline
		for (x = 0; x < theWidth; x++)
		{
			// Grab the pixel
			thePixel = GetPixel(theDC, x, y);
			
			// Fill in the alpha byte (inverted mask)
			dstPtr[(x * 4) + 3] = 255-((thePixel >>  0) & 0x000000FF);
		}

		// Move on to the next scanline
		dstPtr += dstRowBytes;
	}
	
	REALUnlockObject((REALobject)graphics);
	
	return true;
}